<?php

namespace Hepsipay\Payment\Controller\Request;

use Hepsipay\Payment\Helper\ApiService;
use Magento\Customer\Model\Session;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Json\EncoderInterface;
use Magento\Framework\View\Result\PageFactory;
use Magento\Store\Model\StoreManagerInterface;

class CheckoutForm extends Action {
	protected $_context;
	protected $_pageFactory;
	protected $_jsonEncoder;
	protected $_checkoutSession;
	protected $_customerSession;
	protected $_scopeConfig;
	protected $_storeManager;

	public function __construct(
		Context $context,
		EncoderInterface $encoder,
		PageFactory $pageFactory,
		\Magento\Checkout\Model\Session $checkoutSession,
		Session $customerSession,
		ScopeConfigInterface $scopeConfig,
		StoreManagerInterface $storeManager
	) {
		$this->_context = $context;
		$this->_pageFactory = $pageFactory;
		$this->_jsonEncoder = $encoder;
		$this->_checkoutSession = $checkoutSession;
		$this->_customerSession = $customerSession;
		$this->_scopeConfig = $scopeConfig;
		$this->_storeManager = $storeManager;
		parent::__construct( $context );
	}

	public function execute() {

		/* customer to checkout session */

		$jsonData = $this->getRequest()->getContent();
		$postData = json_decode( $jsonData, true );

		$checkoutSession = $this->_checkoutSession->getQuote();

		$environment = $this->_scopeConfig->getValue( 'payment/hepsipay/environment' );
		$merchantNo = $this->_scopeConfig->getValue( 'payment/hepsipay/merchant_no' );
		$terminalNo = $this->_scopeConfig->getValue( 'payment/hepsipay/terminal_no' );
		$storeKey = $this->_scopeConfig->getValue( 'payment/hepsipay/store_key' );

		if ( isset( $postData['quoteEmail'] ) && isset( $postData['quoteId'] ) ) {
			$this->_checkoutSession->setGuestQuoteId( (string) $postData['quoteId'] );
			$this->_customerSession->setEmail( (string) $postData['quoteEmail'] );
		}

		$apiService = new ApiService( $environment, $merchantNo, $terminalNo, $storeKey );

		$billingAddress = $this->_checkoutSession->getQuote()->getBillingAddress();
		$shippingAddress = $this->_checkoutSession->getQuote()->getShippingAddress();

		$order_id = $checkoutSession->getId();

		$amount = round( $checkoutSession->getGrandTotal(), 2 ) * 100;

		if ( $billingAddress && $billingAddress->getTelephone() ) {
			$telephone = $billingAddress->getTelephone();
		} elseif ( $shippingAddress && $shippingAddress->getTelephone() ) {
			$telephone = $shippingAddress->getTelephone();
		} else {
			$telephone = null;
		}

		$customer_phone = (int) preg_replace(
			"/\D/",
			"",
			$telephone
		);

		$customer_phone = substr( $customer_phone, 0, 2 ) !== "90" ? "90" . $customer_phone : $customer_phone;

		$apiService->set_account_key(
			(string) $customer_phone
		);

		$apiService->set_amount( $amount );

		$callBack = $this->_storeManager->getStore()->getBaseUrl();
		$callBack .= "Hepsipay_Payment/response/callback";

		$apiService->set_merchant_callback_url( $callBack );
		$apiService->set_merchant_order_number( $order_id );
		$apiService->set_is_guest_user( true );
		$apiService->set_force_3ds( true );
		$apiService->set_allowed_installments( [
			1,
			2,
			3,
			4,
			5,
			6,
			7,
			8,
			9,
			10,
			11,
			12,
		] );

		$apiService->set_frame_type( 1 );

		$apiService->set_basket_item( [
			"Product"                => "Ürün Toplamı",
			"Price"                  => $amount,
			"SubMerchantMemberId"    => null,
			"SubMerchantMemberPrice" => null,
			"CategoryId"             => null,
			"ExternalId"             => $order_id,
		] );

		try {
			// Check Availability
			$frame_check_availability_response = $apiService->check_availability();

			if ( $frame_check_availability_response->Success === true ) {
				$frame_init_response = $apiService->frame_init();

				if ( $frame_init_response && $frame_init_response->Success === true ) {
					$frame_url = $frame_init_response->FrameUrl;
					$data["verified"] = true;
					$data["frame_url"] = $frame_url;
					$data["callback_url"] = $callBack;
				} else {
					$data["text_description"] =
						$frame_init_response->Message;
				}

			} else {
				$data["text_description"] =
					$frame_check_availability_response->Message;
			}
		} catch ( InvalidArgumentException $e ) {
			$data["text_description"] = (string) $e->getMessage();
		}

		$this->getResponse()->representJson( json_encode( $data ) );

	}
}
