<?php

namespace Hepsipay\Payment\Controller\Response;

use Exception;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\UrlInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Message\ManagerInterface;

use Magento\Quote\Model\QuoteManagement;

use Magento\Framework\App\Action\HttpPostActionInterface as HttpPostActionInterface;
use Hepsipay\Payment\Helper\ApiService;
use Psr\Log\LoggerInterface;

use Magento\Framework\Json\EncoderInterface;
use Magento\Framework\View\Result\PageFactory;
use Magento\Checkout\Model\Session;
use Magento\Quote\Model\Quote;
use Magento\Quote\Api\CartManagementInterface;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Framework\App\Request\Http;
use Magento\Checkout\Api\GuestPaymentInformationManagementInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Quote\Model\QuoteIdMaskFactory;

class Callback extends Action implements HttpPostActionInterface {
	protected $resultJsonFactory;
	protected $orderRepository;
	protected $_context;
	protected $urlBuilder;
	protected $_scopeConfig;
	protected $_messageManager;
	protected $logger;

	protected $_pageFactory;
	protected $_jsonEncoder;
	protected $_checkoutSession;
	protected $_customerSession;
	protected $_quote;
	protected $_cartManagement;
	protected $_resultRedirect;
	protected $_quoteRepository;
	protected $_quoteManagement;
	protected $_request;
	protected $_guestCartManagement;
	protected $_storeManager;
	protected $quoteIdMaskFactory;

	/*
	 *
	 */

	public function __construct(
		Context $context,
		JsonFactory $resultJsonFactory,
		OrderRepositoryInterface $orderRepository,
		UrlInterface $urlBuilder,
		ScopeConfigInterface $scopeConfig,
		ManagerInterface $messageManager,
		EncoderInterface $encoder,
		PageFactory $pageFactory,
		Session $checkoutSession,
		\Magento\Customer\Model\Session $customerSession,
		Quote $quote,
		CartManagementInterface $cartManagement,
		ResultFactory $resultFactory,
		CartRepositoryInterface $quoteRepository,
		QuoteManagement $quoteManagement,
		Http $request,
		GuestPaymentInformationManagementInterface $guestCartManagement,
		StoreManagerInterface $storeManager,
		QuoteIdMaskFactory $quoteIdMaskFactory,
		LoggerInterface $logger
	) {
		$this->resultJsonFactory = $resultJsonFactory;
		$this->orderRepository = $orderRepository;
		$this->_resultRedirect = $context->getResultFactory();
		$this->urlBuilder = $urlBuilder;
		$this->_scopeConfig = $scopeConfig;
		$this->_messageManager = $messageManager;
		$this->_pageFactory = $pageFactory;
		$this->_jsonEncoder = $encoder;
		$this->_checkoutSession = $checkoutSession;
		$this->_customerSession = $customerSession;
		$this->_quote = $quote;
		$this->_cartManagement = $cartManagement;
		$this->_quoteRepository = $quoteRepository;
		$this->_quoteManagement = $quoteManagement;

		$this->_request = $request;
		$this->_guestCartManagement = $guestCartManagement;
		$this->_storeManager = $storeManager;
		$this->quoteIdMaskFactory = $quoteIdMaskFactory;
		$this->logger = $logger;

		parent::__construct( $context );
	}

	protected function _processUrlKeys() {
		return true;
	}

	public function execute() {
		// Gelen verileri al

		parse_str( $this->getRequest()->getContent(), $postData );

		$request = $this->getRequest()->getContent();

		if ( ! $request ) {
			return $this->resultJsonFactory->create()->setData( [ 'success' => false ] );
		}

		$postData = json_decode( $request );

		if ( ! is_object( $postData ) || ! isset( $postData->token ) ) {
			return $this->resultJsonFactory->create()->setData( [ 'success' => false ] );
		}

		$environment = $this->_scopeConfig->getValue( 'payment/hepsipay/environment' );
		$merchantNo = $this->_scopeConfig->getValue( 'payment/hepsipay/merchant_no' );
		$terminalNo = $this->_scopeConfig->getValue( 'payment/hepsipay/terminal_no' );
		$storeKey = $this->_scopeConfig->getValue( 'payment/hepsipay/store_key' );

		$apiService = new ApiService( $environment, $merchantNo, $terminalNo, $storeKey );

		$payment_status = $apiService->payment_status( $postData->token );

		if ( ! is_object( $payment_status ) ) {
			return $this->resultJsonFactory->create()->setData( [ 'success' => false ] );
		}

		if ( in_array( $payment_status->Status, [ 4, 5, 12, 14 ] ) ) {
			// Başarılı ödeme

			$notes[] = "Ödeme Durumu: " . $payment_status->StatusDescription;
			$notes[] = "Ödeme Türü: " . $payment_status->PaymentTypeDescription;
			$notes[] = "Taksit: " . $payment_status->InstallmentCount;
			$notes[] = "Vade Farkı: " . ( $payment_status->InterestAmount ) . " TL";
			$notes[] = "Tahsil Edilen Toplam Tutar: " . ( $payment_status->Amount / 100 ) . " TL";
			$notes[] = "PaymentID: " . $payment_status->PaymentId;

			$orderId = $this->processOrderStatus( $postData->MerchantOrderNumber );

			if ( $orderId ) {

				$order = $this->orderRepository->get( $orderId );

				$order->setState( Order::STATE_PROCESSING );
				$order->setStatus( Order::STATE_PROCESSING );
				$order->addCommentToStatusHistory( implode( " | ", $notes ) );

				$this->orderRepository->save( $order );

				return $this->resultJsonFactory->create()->setData(
					[
						'success'      => true,
						'redirect_url' => $this->urlBuilder->getUrl(
							'checkout/onepage/success',
							[ '_secure' => true ]
						)
					] );
			} else {
				return $this->resultJsonFactory->create()->setData(
					[
						'success'      => false,
						'message'      => "Ödeme Başarısız",
						'redirect_url' => $this->urlBuilder->getUrl(
							'checkout/cart',
							[ '_secure' => true ]
						)
					] );
			}
		}

		$this->_messageManager->addError( $payment_status->UserMessage );

		return $this->resultJsonFactory->create()->setData(
			[
				'success'      => false,
				'message'      => "Payment status doğrulanamadı",
				'redirect_url' => $this->urlBuilder->getUrl(
					'checkout/cart',
					[ '_secure' => true ]
				)
			] );

	}

	protected function processOrderStatus( $quoteId ) {
		try {

			$this->_quote = $this->_quoteRepository->get( $quoteId );

			if ( $this->_customerSession->isLoggedIn() ) {
				/* Place Order - Login Checkout */
				$orderId = $this->_cartManagement->placeOrder( $quoteId, $this->_quote->getPayment() );

			} else {
				/* Place Order - Guest Checkout */

				$quoteIdMask = $this->quoteIdMaskFactory->create()->load( $quoteId, 'quote_id' );
				$quoteId = $quoteIdMask->getMaskedId();

				$orderId = $this->_guestCartManagement->savePaymentInformationAndPlaceOrder( $quoteId,
					$this->_customerSession->getEmail(), $this->_quote->getPayment(),
					$this->_checkoutSession->getBillingAddress() );
			}

			return $orderId;

		} catch ( Exception $e ) {
			// Genel hatalar

			$this->logger->critical( __( 'An error occurred: %1', $e->getMessage() ) );

			return false;
		}
	}

}
