define(
    [
        'Magento_Checkout/js/view/payment/default',
        'jquery',
        'mage/url',
        'Magento_Checkout/js/model/full-screen-loader',
        'Magento_Checkout/js/model/quote',
        'Magento_Customer/js/model/customer',
        'mage/storage',
        'Magento_Checkout/js/model/place-order',
        'Magento_Checkout/js/model/url-builder',
        'uiComponent',
    ],
    function (Component, $, urlBuilder, fullscreenLoader, quote, customer, storage, placeOrderService, mageUrlBuilder) {
        'use strict';
        return Component.extend({
            defaults: {
                template: 'Hepsipay_Payment/payment/hepsipay',
            },
            getInstructions: function () {
                return window.checkoutConfig.payment.instructions[this.item.method];
            },
            getData: function () {
                return {
                    method: this.item.method,
                };
            },

            initializeJsIntegration: function () {
                var quoteEmail, guestQuoteId, checkoutStatus = false;
                var self = this;

                $(document).ready(function () {

                    if (!customer.isLoggedIn()) {
                        quoteEmail = quote.guestEmail;
                        guestQuoteId = quote.getQuoteId();
                    }

                    if (checkoutStatus == false) {

                        $.ajax({
                            url: urlBuilder.build("Hepsipay_Payment/request/checkoutform"),
                            data: JSON.stringify({quoteEmail: quoteEmail, quoteId: guestQuoteId}),
                            type: "POST",
                            headers: {
                                "Content-Type": "application/json"
                            },
                            dataType: "json"
                        }).done(function (data) {

                            $("#loadingBar").hide();

                            if (data && data.frame_url) {
                                self.fire_frame(data.frame_url);
                                checkoutStatus = true;
                            } else {
                                alert("Payment could not be processed. Please try again.");
                            }
                        }).fail(function (jqXHR, textStatus, errorThrown) {
                            console.error("AJAX request failed:", textStatus, errorThrown);
                            alert("Failed to communicate with the server. Please try again later.");
                        });

                    }
                });

                window.addEventListener("message", self.handle_message_events);


            },

            fire_frame: function (frame_url) {
                if (!frame_url) return;
                const arg = {
                    frameUrl: frame_url,
                    frameDivId: "hepsipayFrameInit",
                    paymentStatusCallback: (status) => {
                    },
                    frameFailureCallback: (status) => {
                    },
                    onPaymentSuccessCallback: (status) => {
                    },
                    maxHeight: "800",
                    disableDynamicHeight: false,
                    showFramePaymentButton: false,
                    useApiCallOnSuccessCallback: false,
                    // selectedPaymentInfoCallback: callback,
                    overrideFontFamily: "Inter",
                    hideHeader: false,
                };
                HepsipaySdk.init(arg);
            },

            paymentFormSubmit: function () {
                HepsipaySdk.initPayment();
            },

            handle_message_events: function (event) {
                var self = this;
                const {messageType, ...data} = event.data;
                switch (messageType) {
                    case "hp-payment-available-status":
                        break;
                    case "hp-restart-frame":
                        // self.get_frame_options();
                        break;
                    case "hp-payment-success":

                        $("#hepsipayFrameInit").hide();
                        $("#hepsipay_loader").fadeIn("fast");

                        $.ajax({
                            url: urlBuilder.build("Hepsipay_Payment/response/callback"),
                            data: JSON.stringify(data),
                            type: "post",
                            headers:{
                                "Content-Type": 'application/json'
                            },
                            dataType: "json"
                        }).done(function (response) {
                            if(response?.success===true){
                                window.location = response?.redirect_url;
                            }

                        });



                        break;
                }
            },


        });
    }
);
